
This section contains lab-specific code-style rules that go beyond the general discussion in *Code and Data*.

### Be a good code citizen

Team members should take the time to improve code they are modifying or extending even if they did not write it themselves. A core of good code plus a long series of edits and accretions equals bad code. The problem is that the logical structure that made sense for the program when it was small no longer makes sense as it grows. It is critical to regularly look at the program as a whole and improve the logical structure through reorganization and abstraction. Programmers call this “refactoring.” Even if your immediate task only requires modifying a small part of a program, we encourage you to take the time to improve the program more broadly. At a minimum, you should guarantee that the code quality of the program overall is at least as good as it was when you started. A resource on how to implement refactoring can be found [here](https://refactoring.guru/refactoring/).

### Keep it short

No line of code should be more than 100 characters long. All languages we work in allow you to break a logical line across multiple lines on the page (e.g, using `///` in Stata or `...` in Matlab). You may want to set your editor to show a “margin” at 100 characters.

Functions should not typically be longer than 200 lines.

### Use white space and indents to make code scannable

Carriage returns should separate blocks of related commands. Major sections of code can be separated with two carriage returns.

Use spaces rather than tab characters to indent. Many editors automatically insert spaces when you press tab. Others allow this as an option.

Commands inside functions or loops should be indented four spaces.

The first row of every function definition should start at zero indent. Do not use indents to indicate levels of the functional hierarchy (functions, sub-functions, sub-sub-functions, etc.)

Logical lines broken across multiple rows (e.g, using `///` in Stata or `...` in Matlab) should be indented two spaces and aligned vertically for clarity. E.g.,

```{.R}
var inset = {  
  top:    10,   
  right:  20,   
  bottom: 15,   
  left:   12 
};
```

or

```{.R}
function [data, options] = format_raw_data(
  str dataset_name,
  int population,
  double density,
  struct options,
)
```

Equals signs, arithmetic operators, etc. should be padded with spaces where it improves clarity. Function arguments should be separated with a space after the comma:

```{.R}
my_function(x, y, z)
```

rather than

```{.R}
my_function(x,y,z)
```

By default, use this form for braces defining blocks of code

```{.R}
for i = 1:5 {
    ...
}
```

rather than this form

```{.R}
for i = 1:5 {
    ...
    }
```

A large number of related commands that suggests a tabular format should be aligned vertically for easy scanning. E.g.,

```{.R}
cats                  = 10;
dogs                  = 4;
tyrannosaurus_rexes   = 1;
pigeons               = 2;
pigs                  = 8
dragons               = 25;
other_animals         = 5;
```

### Order functions for linear reading

Remember to order functions for linear reading. E.g.,

```{.R}
function main
    [code that calls func_I and func_II]
function func_I
    [code that calls func_1]
function func_1
    [...]
function func_II
    [code that calls func_2]
function func_2
    [...]
```

The alternative ordering would be by function hierarchy: `main`, `func_I`, `func_II`, `func_1`, `func_2`. This arrangement makes that levels at which the different functions live clearer, but is harder to read linearly. It is possible that a script is so long, and has so many levels of functional hierarchy, that nobody would ever read it linearly. In this case, one could argue that arranging by level makes more sense. But the correct response in this case is usually to break the script into multiple files and preserver the property that each individual file can be read beginning-to-end.


### Object Oriented Programming


Use overriding methods instead of switches where appropriate.

For most projects we use object-oriented programming (OOP). In this paradigm we define classes of objects which have associated properties and methods. Instead of, say, creating several separate variables and defining a function to operate on them, we can define an object class which has the variables as properties and the function as a method. We can also define subclasses, which inherit all the properties and methods of the superclass except those explicitly changed in the subclass file.

There are several reasons why we might want to use OOP. One reason is that creating classes and subclasses is a much clearer way of coding a model with several different variants than writing a program with a lot of if/else switches.

For example, suppose we have a class Model with a method DrawRandVar which draws a random variable from a certain distribution. Suppose that usually we want the variable to be drawn from the standard normal distribution, but we also want to look at results which require it to be drawn from a uniform distribution. We could code this as an option switch:

```{.R}
function draw = DrawRandVar(obj, option) 
    if option == ’uniform’ 
        draw = rand;
    else 
        draw = randn;
    end
end
```

However, we could also create a subclass UniformModel of the class Model which contains the overriding method:

```{.R}
function draw = DrawRandVar(obj) 
    draw = rand;
end
```

If you are working with a model coded in OOP, you should think carefully about whether to include switches or write subclasses with overriding methods. Code with lots of switches can become harder to follow and parse, and creates a false sense of generality. For example, if we add an option switch to this method we may need to add it to others to make the model consistent; this would be easier to achieve using a subclass. Furthermore, switches are checked every time a method is called, unnecessarily increasing the computational cost of running the method.

Note that when writing subclasses, you should be careful to keep superclass methods intuitive and "shy." For example, if a superclass method takes an input which is redundant in the superclass and all but one of several subclasses, it may be better to refactor to avoid this redundancy. Having arguments which are redundant in the superclass makes it more difficult for another user to understand what the superclass method does. On the other hand, if subclass methods only modify one or two lines of a long superclass method, you should think about abstracting out these lines as a separate method in the superclass. This new method can then be modified in the subclass, avoiding the redundancy introduced by copying a long method. In general, you should think about structuring superclass methods and their corresponding subclass methods in the most intuitive way possible.
