
`SCons` is a software construction tool written in python. We use `SCons` to execute the scripts in a directory and to track dependencies between files. One should think of the directory as a directed acyclic graph (DAG), where files are the nodes and dependencies are the edges. The acyclic nature of the dependency structure is crucial.  

In the GSLab template, we have two SCons subdirectories. At the root of a SCons subdirectory, we have an `SConstruct` file. `SConstruct` sets the environment for later scripts and is the head of the DAG. `SConstruct` has two key outputs. In `./release`, it produces an `sconstruct.log` with the exit status of each `SConscript` that is run during a given execution. In the root of a SCons subdirectory, it outputs `.sconsign.dblite`. This file stores the MD5 hash and time-stamp at the end of the directory's execution for each file that is being tracked (i.e., all files specified as 'targets' or 'source' -- see below for definitions). 

Each subdirectory of a Scons subdirectory with code that is to be executed should contain an `Sconscript` file. The `Sconscript` requires three fields for each executable in a subdirectory. 
 - The `target` field gives the locations of all output that we want to track as being dependent on a given command. 
   - `.log` files should __not__ be specified as targets.
 - The `source` field gives the locations of all input that is used to create the output of a command. 
   - `.log` files should __not__ be specified as source.
 - The `action` field gives the command necessary to execute the script. 

Each `SConscript` file writes to a single `sconscript.log` in the parallel location.

Example of a command within `SConscript`:
```
env.Command(
	target = '#output/analysis/plot.eps', 
	source = ['#output/data/data.txt', 'analysis.do'],
	action = '''statamp -e source/analysis/analysis.do && \
				mv analysis.log output/analysis/sconscript.log'''
	)
```
	
The sub-DAG for this command would look like:
	
	data.txt 
	        \
	         \ ---> statamp ----> plot.eps
	         /
	        /
	analysis.do       
	
If data.txt, analysis.do, or plot.eps have changed, then this command will re-run when `scons` is entered.


The reason for using SCons can be clearly seen in the following example:

Let's assume `File B` is a target and `File A` is a source for some command. Likewise, `File Y` is a target and file `File Z` is a source for some other command. After a successful run of `scons`, we discover a mistake in `File A`. After fixing the mistake, we can run `scons` which will only re-build `File B`, because only `File B` is downstream of `File A` in the DAG. 

To run:

  *  The entire directory:
      *  In the root directory, type 'scons'. This should run everything that is flagged as being modified or with dependencies that have been modified.
  
  *  A single directory of targets:
      * `scons output/data` will re-build the `output/data` folder if it is out of sync, without rebuilding other files.
 
  *  A single target file:
      * `scons output/paper/paper.pdf` will re-run only the code needed to update `output/paper/paper.pdf` without rebuilding other files.

Other principles:

  *  We must set `env = Environment(ENV = {'PATH' : os.environ['PATH']}, IMPLICIT_COMMAND_DEPENDENCIES = 0)`
     *  `IMPLICIT_COMMAND_DEPENDENCIES = 0` tells SCons to not track the executables of any commands used, which will be different across machines.
    *  `ENV = {'PATH' : os.environ['PATH']}` gives SCons access to the PATH environment variable.

  *  We should only be calculating the MD5 hash if the timestamp has changed (e.g., `env.Decider('MD5-timestamp')`), otherwise we waste time on computing the hash when it is not needed. This is especially important when we have large files.

  *  Do not commit `.sconsign.dblite`. Otherwise the information it stores may fall out of sync with the actual state of the directory when someone new clones the repository. 
